@@ -7,6 +7,7 @@ searching etc. See :ref:`walking` for details.
 
 from __future__ import unicode_literals
 
+import typing
 from collections import defaultdict
 from collections import deque
 from collections import namedtuple
@@ -19,58 +20,81 @@ from .path import abspath
 from .path import join
 from .path import normpath
 
+if False:  # typing.TYPE_CHECKING
+    from typing import (
+        Any,
+        Callable,
+        Collection,
+        Iterator,
+        List,
+        Optional,
+        MutableMapping,
+        Text,
+        Tuple,
+        Type,
+    )
+    from .base import FS
+    from .info import Info
 
-Step = namedtuple('Step', 'path, dirs, files')
+    OnError = Callable[[Text, Exception], bool]
+
+
+_F = typing.TypeVar("_F", bound="FS")
+
+
+Step = namedtuple("Step", "path, dirs, files")
 """type: a *step* in a directory walk.
 """
 
 
+# TODO(@althonos): It could be a good idea to create an Abstract Base Class
+#                  BaseWalker (with methods walk, files, dirs and info) ?
+
+
 class Walker(object):
     """A walker object recursively lists directories in a filesystem.
 
     Arguments:
-        ignore_errors (bool, optional): If `True`, any errors reading
-            a directory will be ignored, otherwise exceptions will be
-            raised.
+        ignore_errors (bool): If `True`, any errors reading a
+            directory will be ignored, otherwise exceptions will
+            be raised.
         on_error (callable, optional): If ``ignore_errors`` is `False`,
             then this callable will be invoked for a path and the exception
             object. It should return `True` to ignore the error, or `False`
             to re-raise it.
-        search (str, optional): If ``'breadth'`` then the directory will be
+        search (str): If ``'breadth'`` then the directory will be
             walked *top down*. Set to ``'depth'`` to walk *bottom up*.
-        filter (list, optional): If supplied, this parameter should be a
-            list of filename patterns, e.g. ``['*.py']``. Files will only
-            be returned if the final component matches one of the patterns.
-        exclude_dirs (list, optional): A list of patterns that will be used
-            to filter out directories from the walk. e.g.
+        filter (list, optional): If supplied, this parameter should be
+            a list of filename patterns, e.g. ``['*.py']``. Files will
+            only be returned if the final component matches one of the
+            patterns.
+        exclude_dirs (list, optional): A list of patterns that will be
+            used to filter out directories from the walk. e.g.
             ``['*.svn', '*.git']``.
         max_depth (int, optional): Maximum directory depth to walk.
 
     """
 
-    def __init__(self,
-                 ignore_errors=False,
-                 on_error=None,
-                 search="breadth",
-                 filter=None,
-                 exclude_dirs=None,
-                 max_depth=None):
-        if search not in ('breadth', 'depth'):
+    def __init__(
+        self,
+        ignore_errors=False,  # type: bool
+        on_error=None,  # type: Optional[OnError]
+        search="breadth",  # type: Text
+        filter=None,  # type: Optional[List[Text]]
+        exclude_dirs=None,  # type: Optional[List[Text]]
+        max_depth=None,  # type: Optional[int]
+    ):
+        # type: (...) -> None
+        if search not in ("breadth", "depth"):
             raise ValueError("search must be 'breadth' or 'depth'")
         self.ignore_errors = ignore_errors
         if on_error:
             if ignore_errors:
-                raise ValueError(
-                    'on_error is invalid when ignore_errors==True'
-                )
+                raise ValueError("on_error is invalid when ignore_errors==True")
         else:
-            on_error = (
-                self._ignore_errors
-                if ignore_errors
-                else self._raise_errors
-            )
+            on_error = self._ignore_errors if ignore_errors else self._raise_errors
         if not callable(on_error):
-            raise TypeError('on_error must be callable')
+            raise TypeError("on_error must be callable")
 
         self.on_error = on_error
         self.search = search
@@ -81,24 +105,28 @@ class Walker(object):
 
     @classmethod
     def _ignore_errors(cls, path, error):
+        # type: (Text, Exception) -> bool
         """Default on_error callback."""
         return True
 
     @classmethod
     def _raise_errors(cls, path, error):
+        # type: (Text, Exception) -> bool
         """Callback to re-raise dir scan errors."""
         return False
 
     @classmethod
     def _calculate_depth(cls, path):
+        # type: (Text) -> int
         """Calculate the 'depth' of a directory path (number of
         components).
         """
-        _path = path.strip('/')
-        return _path.count('/') + 1 if _path else 0
+        _path = path.strip("/")
+        return _path.count("/") + 1 if _path else 0
 
     @classmethod
     def bind(cls, fs):
+        # type: (_F) -> BoundWalker[_F]
         """Bind a `Walker` instance to a given filesystem.
 
         This *binds* in instance of the Walker to a given filesystem, so
@@ -134,32 +162,40 @@ class Walker(object):
         return BoundWalker(fs)
 
     def __repr__(self):
+        # type: () -> Text
         return make_repr(
             self.__class__.__name__,
             ignore_errors=(self.ignore_errors, False),
             on_error=(self.on_error, None),
-            search=(self.search, 'breadth'),
+            search=(self.search, "breadth"),
             filter=(self.filter, None),
             exclude_dirs=(self.exclude_dirs, None),
-            max_depth=(self.max_depth, None)
+            max_depth=(self.max_depth, None),
         )
 
-    def _iter_walk(self, fs, path, namespaces=None):
+    def _iter_walk(
+        self,
+        fs,  # type: FS
+        path,  # type: Text
+        namespaces=None,  # type: Optional[Collection[Text]]
+    ):
+        # type: (...) -> Iterator[Tuple[Text, Optional[Info]]]
         """Get the walk generator."""
-        if self.search == 'breadth':
+        if self.search == "breadth":
             return self._walk_breadth(fs, path, namespaces=namespaces)
         else:
             return self._walk_depth(fs, path, namespaces=namespaces)
 
     def _check_open_dir(self, fs, path, info):
+        # type: (FS, Text, Info) -> bool
         """Check if a directory should be considered in the walk.
         """
-        if (self.exclude_dirs is not None and
-                fs.match(self.exclude_dirs, info.name)):
+        if self.exclude_dirs is not None and fs.match(self.exclude_dirs, info.name):
             return False
         return self.check_open_dir(fs, path, info)
 
     def check_open_dir(self, fs, path, info):
+        # type: (FS, Text, Info) -> bool
         """Check if a directory should be opened.
 
         Override to exclude directories from the walk.
@@ -176,12 +212,14 @@ class Walker(object):
         return True
 
     def _check_scan_dir(self, fs, path, info, depth):
+        # type: (FS, Text, Info, int) -> bool
         """Check if a directory contents should be scanned."""
         if self.max_depth is not None and depth >= self.max_depth:
             return False
         return self.check_scan_dir(fs, path, info)
 
     def check_scan_dir(self, fs, path, info):
+        # type: (FS, Text, Info) -> bool
         """Check if a directory should be scanned.
 
         Override to omit scanning of certain directories. If a directory
@@ -200,6 +238,7 @@ class Walker(object):
         return True
 
     def check_file(self, fs, info):
+        # type: (FS, Info) -> bool
         """Check if a filename should be included.
 
         Override to exclude files from the walk.
@@ -214,7 +253,13 @@ class Walker(object):
         """
         return fs.match(self.filter, info.name)
 
-    def _scan(self, fs, dir_path, namespaces):
+    def _scan(
+        self,
+        fs,  # type: FS
+        dir_path,  # type: Text
+        namespaces=None,  # type: Optional[Collection[Text]]
+    ):
+        # type: (...) -> Iterator[Info]
         """Get an iterator of `Info` objects for a directory path.
 
         Arguments:
@@ -235,12 +280,18 @@ class Walker(object):
             if not self.on_error(dir_path, error):
                 six.reraise(type(error), error)
 
-    def walk(self, fs, path='/', namespaces=None):
+    def walk(
+        self,
+        fs,  # type: FS
+        path="/",  # type: Text
+        namespaces=None,  # type: Optional[Collection[Text]]
+    ):
+        # type: (...) -> Iterator[Step]
         """Walk the directory structure of a filesystem.
 
         Arguments:
             fs (FS): A filesystem instance.
-            path (str, optional): A path to a directory on the filesystem.
+            path (str): A path to a directory on the filesystem.
             namespaces (list, optional): A list of additional namespaces
                 to add to the `Info` objects.
 
@@ -264,12 +315,12 @@ class Walker(object):
 
         """
         _path = abspath(normpath(path))
-        dir_info = defaultdict(list)
+        dir_info = defaultdict(list)  # type: MutableMapping[Text, List[Info]]
         _walk = self._iter_walk(fs, _path, namespaces=namespaces)
         for dir_path, info in _walk:
             if info is None:
-                dirs = []
-                files = []
+                dirs = []  # type: List[Info]
+                files = []  # type: List[Info]
                 for _info in dir_info[dir_path]:
                     (dirs if _info.is_dir else files).append(_info)
                 yield Step(dir_path, dirs, files)
@@ -277,12 +328,13 @@ class Walker(object):
             else:
                 dir_info[dir_path].append(info)
 
-    def files(self, fs, path='/'):
+    def files(self, fs, path="/"):
+        # type: (FS, Text) -> Iterator[Text]
         """Walk a filesystem, yielding absolute paths to files.
 
         Arguments:
             fs (FS): A filesystem instance.
-            path (str, optional): A path to a directory on the filesystem.
+            path (str): A path to a directory on the filesystem.
 
         Yields:
             str: absolute path to files on the filesystem found
@@ -293,12 +345,13 @@ class Walker(object):
             if info is not None and not info.is_dir:
                 yield join(_path, info.name)
 
-    def dirs(self, fs, path='/'):
+    def dirs(self, fs, path="/"):
+        # type: (FS, Text) -> Iterator[Text]
         """Walk a filesystem, yielding absolute paths to directories.
 
         Arguments:
             fs (FS): A filesystem instance.
-            path (str, optional): A path to a directory on the filesystem.
+            path (str): A path to a directory on the filesystem.
 
         Yields:
             str: absolute path to directories on the filesystem found
@@ -309,12 +362,18 @@ class Walker(object):
             if info is not None and info.is_dir:
                 yield join(_path, info.name)
 
-    def info(self, fs, path='/', namespaces=None):
+    def info(
+        self,
+        fs,  # type: FS
+        path="/",  # type: Text
+        namespaces=None,  # type: Optional[Collection[Text]]
+    ):
+        # type: (...) -> Iterator[Tuple[Text, Info]]
         """Walk a filesystem, yielding tuples of ``(<path>, <info>)``.
 
         Arguments:
             fs (FS): A filesystem instance.
-            path (str, optional): A path to a directory on the filesystem.
+            path (str): A path to a directory on the filesystem.
             namespaces (list, optional): A list of additional namespaces
                 to add to the `Info` objects.
 
@@ -327,7 +386,13 @@ class Walker(object):
             if info is not None:
                 yield join(_path, info.name), info
 
-    def _walk_breadth(self, fs, path, namespaces=None):
+    def _walk_breadth(
+        self,
+        fs,  # type: FS
+        path,  # type: Text
+        namespaces=None,  # type: Optional[Collection[Text]]
+    ):
+        # type: (...) -> Iterator[Tuple[Text, Optional[Info]]]
         """Walk files using a *breadth first* search.
         """
         queue = deque([path])
@@ -349,17 +414,26 @@ class Walker(object):
                         yield dir_path, info  # Found a file
             yield dir_path, None  # End of directory
 
-    def _walk_depth(self, fs, path, namespaces=None):
+    def _walk_depth(
+        self,
+        fs,  # type: FS
+        path,  # type: Text
+        namespaces=None,  # type: Optional[Collection[Text]]
+    ):
+        # type: (...) -> Iterator[Tuple[Text, Optional[Info]]]
         """Walk files using a *depth first* search.
         """
         # No recursion!
 
         def scan(path):
+            # type: (Text) -> Iterator[Info]
             """Perform scan."""
             return self._scan(fs, path, namespaces=namespaces)
 
+        stack = [
+            (path, scan(path), None)
+        ]  # type: List[Tuple[Text, Iterator[Info], Optional[Tuple[Text, Info]]]]
         depth = self._calculate_depth(path)
-        stack = [(path, scan(path), None)]
         push = stack.append
 
         while stack:
@@ -383,12 +457,12 @@ class Walker(object):
                     yield dir_path, info
 
 
-class BoundWalker(object):
+class BoundWalker(typing.Generic[_F]):
     """A class that binds a `Walker` instance to a `FS` instance.
 
     Arguments:
         fs (FS): A filesystem instance.
-        walker_class (type, optional): A `~fs.walk.WalkerBase`
+        walker_class (type): A `~fs.walk.WalkerBase`
             sub-class. The default uses `~fs.walk.Walker`.
 
     You will typically not need to create instances of this class
@@ -407,26 +481,32 @@ class BoundWalker(object):
     """
 
     def __init__(self, fs, walker_class=Walker):
+        # type: (_F, Type[Walker]) -> None
         self.fs = fs
         self.walker_class = walker_class
 
     def __repr__(self):
+        # type: () -> Text
         return "BoundWalker({!r})".format(self.fs)
 
     def _make_walker(self, *args, **kwargs):
+        # type: (*Any, **Any) -> Walker
         """Create a walker instance.
         """
         walker = self.walker_class(*args, **kwargs)
         return walker
 
-    def walk(self,
-             path='/',
-             namespaces=None,
-             **kwargs):
+    def walk(
+        self,
+        path="/",  # type: Text
+        namespaces=None,  # type: Optional[Collection[Text]]
+        **kwargs  # type: Any
+    ):
+        # type: (...) -> Iterator[Step]
         """Walk the directory structure of a filesystem.
 
         Arguments:
-            path (str, optional):
+            path (str):
             namespaces (list, optional): A list of namespaces to include
                 in the resource information, e.g. ``['basic', 'access']``
                 (defaults to ``['basic']``).
@@ -473,7 +553,8 @@ class BoundWalker(object):
 
     __call__ = walk
 
-    def files(self, path='/', **kwargs):
+    def files(self, path="/", **kwargs):
+        # type: (Text, **Any) -> Iterator[Text]
         """Walk a filesystem, yielding absolute paths to files.
 
         Arguments:
@@ -499,7 +580,7 @@ class BoundWalker(object):
             max_depth (int, optional): Maximum directory depth to walk.
 
         Returns:
-            ~collections.Iterable: An iterable of file paths (absolute
+            ~collections.Iterator: An iterator over file paths (absolute
             from the filesystem root).
 
         This method invokes `Walker.files` with the bound `FS` object.
@@ -508,7 +589,8 @@ class BoundWalker(object):
         walker = self._make_walker(**kwargs)
         return walker.files(self.fs, path=path)
 
-    def dirs(self, path='/', **kwargs):
+    def dirs(self, path="/", **kwargs):
+        # type: (Text, **Any) -> Iterator[Text]
         """Walk a filesystem, yielding absolute paths to directories.
 
         Arguments:
@@ -530,7 +612,7 @@ class BoundWalker(object):
             max_depth (int, optional): Maximum directory depth to walk.
 
         Returns:
-            ~collections.iterable: an iterable of directory paths
+            ~collections.Iterator: an iterator over directory paths
             (absolute from the filesystem root).
 
         This method invokes `Walker.dirs` with the bound `FS` object.
@@ -539,11 +621,17 @@ class BoundWalker(object):
         walker = self._make_walker(**kwargs)
         return walker.dirs(self.fs, path=path)
 
-    def info(self, path='/', namespaces=None, **kwargs):
+    def info(
+        self,
+        path="/",  # type: Text
+        namespaces=None,  # type: Optional[Collection[Text]]
+        **kwargs  # type: Any
+    ):
+        # type: (...) -> Iterator[Tuple[Text, Info]]
         """Walk a filesystem, yielding path and `Info` of resources.
 
         Arguments:
-            path (str, optional):
+            path (str): A path to a directory.
             namespaces (list, optional): A list of namespaces to include
                 in the resource information, e.g. ``['basic', 'access']``
                 (defaults to ``['basic']``).
@@ -575,11 +663,8 @@ class BoundWalker(object):
 
         """
         walker = self._make_walker(**kwargs)
-        return walker.info(
-            self.fs,
-            path=path,
-            namespaces=namespaces
-        )
+        return walker.info(self.fs, path=path, namespaces=namespaces)
+
 
 # Allow access to default walker from the module
 # For example:
